@@ -1,7 +1,7 @@
 /*
  *  package.c
  * 
- *  Copyright (c) 2002-2003 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2002-2004 by Judd Vinet <jvinet@zeroflux.org>
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -289,6 +289,9 @@ int is_pkgin(pkginfo_t *needle, PMList *haystack)
 {
 	PMList *lp;
 	pkginfo_t *info;
+	if(needle == NULL || haystack == NULL) {
+		return(0);
+	}
 
 	for(lp = haystack; lp; lp = lp->next) {
 		info = (pkginfo_t*)lp->data;
